@@ -9,7 +9,7 @@ class TestFilters < JekyllUnitTest
 
     def initialize(opts = {})
       @site = Jekyll::Site.new(opts.merge("skip_config_files" => true))
-      @context = Liquid::Context.new(@site.site_payload, {}, { :site => @site })
+      @context = Liquid::Context.new(@site.site_payload, {}, :site => @site)
     end
   end
 
@@ -37,23 +37,29 @@ class TestFilters < JekyllUnitTest
   context "filters" do
     setup do
       @sample_time = Time.utc(2013, 3, 27, 11, 22, 33)
-      @filter = make_filter_mock({
+      @timezone_before_test = ENV["TZ"]
+      @filter = make_filter_mock(
         "timezone"               => "UTC",
         "url"                    => "http://example.com",
         "baseurl"                => "/base",
-        "dont_show_posts_before" => @sample_time,
-      })
+        "dont_show_posts_before" => @sample_time
+      )
       @sample_date = Date.parse("2013-03-02")
       @time_as_string = "September 11, 2001 12:46:30 -0000"
       @time_as_numeric = 1_399_680_607
       @integer_as_string = "142857"
       @array_of_objects = [
+        { "color" => "teal", "size" => "large"  },
         { "color" => "red",  "size" => "large"  },
         { "color" => "red",  "size" => "medium" },
         { "color" => "blue", "size" => "medium" },
       ]
     end
 
+    teardown do
+      ENV["TZ"] = @timezone_before_test
+    end
+
     should "markdownify with simple string" do
       assert_equal(
         "<p>something <strong>really</strong> simple</p>\n",
@@ -88,7 +94,7 @@ class TestFilters < JekyllUnitTest
       end
 
       should "escapes special characters when configured to do so" do
-        kramdown = make_filter_mock({ :kramdown => { :entity_output => :symbolic } })
+        kramdown = make_filter_mock(:kramdown => { :entity_output => :symbolic })
         assert_equal(
           "&ldquo;This filter&rsquo;s test&hellip;&rdquo;",
           kramdown.smartify(%q{"This filter's test..."})
@@ -137,14 +143,18 @@ class TestFilters < JekyllUnitTest
 
     should "sassify with simple string" do
       assert_equal(
-        "p {\n  color: #123456; }\n",
-        @filter.sassify("$blue:#123456\np\n  color: $blue")
+        "p { color: #123456; }\n",
+        @filter.sassify(<<~SASS)
+          $blue: #123456
+          p
+            color: $blue
+        SASS
       )
     end
 
     should "scssify with simple string" do
       assert_equal(
-        "p {\n  color: #123456; }\n",
+        "p { color: #123456; }\n",
         @filter.scssify("$blue:#123456; p{color: $blue}")
       )
     end
@@ -406,82 +416,82 @@ class TestFilters < JekyllUnitTest
 
       should "ensure the leading slash for the baseurl" do
         page_url = "about/my_favorite_page/"
-        filter = make_filter_mock({
+        filter = make_filter_mock(
           "url"     => "http://example.com",
-          "baseurl" => "base",
-        })
+          "baseurl" => "base"
+        )
         assert_equal "http://example.com/base/#{page_url}", filter.absolute_url(page_url)
       end
 
       should "be ok with a blank but present 'url'" do
         page_url = "about/my_favorite_page/"
-        filter = make_filter_mock({
+        filter = make_filter_mock(
           "url"     => "",
-          "baseurl" => "base",
-        })
+          "baseurl" => "base"
+        )
         assert_equal "/base/#{page_url}", filter.absolute_url(page_url)
       end
 
       should "be ok with a nil 'url'" do
         page_url = "about/my_favorite_page/"
-        filter = make_filter_mock({
+        filter = make_filter_mock(
           "url"     => nil,
-          "baseurl" => "base",
-        })
+          "baseurl" => "base"
+        )
         assert_equal "/base/#{page_url}", filter.absolute_url(page_url)
       end
 
       should "be ok with a nil 'baseurl'" do
         page_url = "about/my_favorite_page/"
-        filter = make_filter_mock({
+        filter = make_filter_mock(
           "url"     => "http://example.com",
-          "baseurl" => nil,
-        })
+          "baseurl" => nil
+        )
         assert_equal "http://example.com/#{page_url}", filter.absolute_url(page_url)
       end
 
       should "not prepend a forward slash if input is empty" do
         page_url = ""
-        filter = make_filter_mock({
+        filter = make_filter_mock(
           "url"     => "http://example.com",
-          "baseurl" => "/base",
-        })
+          "baseurl" => "/base"
+        )
         assert_equal "http://example.com/base", filter.absolute_url(page_url)
       end
 
       should "not append a forward slash if input is '/'" do
         page_url = "/"
-        filter = make_filter_mock({
+        filter = make_filter_mock(
           "url"     => "http://example.com",
-          "baseurl" => "/base",
-        })
+          "baseurl" => "/base"
+        )
         assert_equal "http://example.com/base/", filter.absolute_url(page_url)
       end
 
       should "not append a forward slash if input is '/' and nil 'baseurl'" do
         page_url = "/"
-        filter = make_filter_mock({
+        filter = make_filter_mock(
           "url"     => "http://example.com",
-          "baseurl" => nil,
-        })
+          "baseurl" => nil
+        )
         assert_equal "http://example.com/", filter.absolute_url(page_url)
       end
 
       should "not append a forward slash if both input and baseurl are simply '/'" do
         page_url = "/"
-        filter = make_filter_mock({
+        filter = make_filter_mock(
           "url"     => "http://example.com",
-          "baseurl" => "/",
-        })
+          "baseurl" => "/"
+        )
         assert_equal "http://example.com/", filter.absolute_url(page_url)
       end
 
       should "normalize international URLs" do
         page_url = ""
-        filter = make_filter_mock({
+        filter = make_filter_mock(
           "url"     => "http://ümlaut.example.org/",
-          "baseurl" => nil,
-        })
+          "baseurl" => nil
+        )
         assert_equal "http://xn--mlaut-jva.example.org/", filter.absolute_url(page_url)
       end
 
@@ -492,19 +502,19 @@ class TestFilters < JekyllUnitTest
 
       should "transform the input URL to a string" do
         page_url = "/my-page.html"
-        filter = make_filter_mock({ "url" => Value.new(proc { "http://example.org" }) })
+        filter = make_filter_mock("url" => Value.new(proc { "http://example.org" }))
         assert_equal "http://example.org#{page_url}", filter.absolute_url(page_url)
       end
 
       should "not raise a TypeError when passed a hash" do
-        assert @filter.absolute_url({ "foo" => "bar" })
+        assert @filter.absolute_url("foo" => "bar")
       end
 
       context "with a document" do
         setup do
-          @site = fixture_site({
-            "collections" => ["methods"],
-          })
+          @site = fixture_site(
+            "collections" => ["methods"]
+          )
           @site.process
           @document = @site.collections["methods"].docs.detect do |d|
             d.relative_path == "_methods/configuration.md"
@@ -531,7 +541,7 @@ class TestFilters < JekyllUnitTest
 
       should "ensure the leading slash for the baseurl" do
         page_url = "about/my_favorite_page/"
-        filter = make_filter_mock({ "baseurl" => "base" })
+        filter = make_filter_mock("baseurl" => "base")
         assert_equal "/base/#{page_url}", filter.relative_url(page_url)
       end
 
@@ -542,61 +552,62 @@ class TestFilters < JekyllUnitTest
 
       should "be ok with a nil 'baseurl'" do
         page_url = "about/my_favorite_page/"
-        filter = make_filter_mock({
+        filter = make_filter_mock(
           "url"     => "http://example.com",
-          "baseurl" => nil,
-        })
+          "baseurl" => nil
+        )
         assert_equal "/#{page_url}", filter.relative_url(page_url)
       end
 
       should "not prepend a forward slash if input is empty" do
         page_url = ""
-        filter = make_filter_mock({
+        filter = make_filter_mock(
           "url"     => "http://example.com",
-          "baseurl" => "/base",
-        })
+          "baseurl" => "/base"
+        )
         assert_equal "/base", filter.relative_url(page_url)
       end
 
       should "not prepend a forward slash if baseurl ends with a single '/'" do
         page_url = "/css/main.css"
-        filter = make_filter_mock({
+        filter = make_filter_mock(
           "url"     => "http://example.com",
-          "baseurl" => "/base/",
-        })
+          "baseurl" => "/base/"
+        )
         assert_equal "/base/css/main.css", filter.relative_url(page_url)
       end
 
       should "not return valid URI if baseurl ends with multiple '/'" do
         page_url = "/css/main.css"
-        filter = make_filter_mock({
+        filter = make_filter_mock(
           "url"     => "http://example.com",
-          "baseurl" => "/base//",
-        })
+          "baseurl" => "/base//"
+        )
         refute_equal "/base/css/main.css", filter.relative_url(page_url)
       end
 
       should "not prepend a forward slash if both input and baseurl are simply '/'" do
         page_url = "/"
-        filter = make_filter_mock({
+        filter = make_filter_mock(
           "url"     => "http://example.com",
-          "baseurl" => "/",
-        })
+          "baseurl" => "/"
+        )
         assert_equal "/", filter.relative_url(page_url)
       end
 
       should "not return the url by reference" do
-        filter = make_filter_mock({ :baseurl => nil })
+        filter = make_filter_mock(:baseurl => nil)
         page = Page.new(filter.site, test_dir("fixtures"), "", "front_matter.erb")
         assert_equal "/front_matter.erb", page.url
         url = filter.relative_url(page.url)
         url << "foo"
+        assert_equal "/front_matter.erb", filter.relative_url(page.url)
         assert_equal "/front_matter.erb", page.url
       end
 
       should "transform the input baseurl to a string" do
         page_url = "/my-page.html"
-        filter = make_filter_mock({ "baseurl" => Value.new(proc { "/baseurl/" }) })
+        filter = make_filter_mock("baseurl" => Value.new(proc { "/baseurl/" }))
         assert_equal "/baseurl#{page_url}", filter.relative_url(page_url)
       end
 
@@ -640,7 +651,7 @@ class TestFilters < JekyllUnitTest
 
     context "jsonify filter" do
       should "convert hash to json" do
-        assert_equal "{\"age\":18}", @filter.jsonify({ :age => 18 })
+        assert_equal "{\"age\":18}", @filter.jsonify(:age => 18)
       end
 
       should "convert array to json" do
@@ -654,6 +665,7 @@ class TestFilters < JekyllUnitTest
       should "convert drop to json" do
         @filter.site.read
         expected = {
+          "name"          => "2008-02-02-published.markdown",
           "path"          => "_posts/2008-02-02-published.markdown",
           "previous"      => nil,
           "output"        => nil,
@@ -687,10 +699,11 @@ class TestFilters < JekyllUnitTest
       should "convert drop with drops to json" do
         @filter.site.read
         actual = @filter.jsonify(@filter.site.to_liquid)
-        assert_equal JSON.parse(actual)["jekyll"], {
+        expected = {
           "environment" => "development",
           "version"     => Jekyll::VERSION,
         }
+        assert_equal expected, JSON.parse(actual)["jekyll"]
       end
 
       # rubocop:disable Style/StructInheritance
@@ -699,12 +712,13 @@ class TestFilters < JekyllUnitTest
           [message]
         end
       end
+
       class T < Struct.new(:name)
         def to_liquid
           {
             "name" => name,
             :v     => 1,
-            :thing => M.new({ :kay => "jewelers" }),
+            :thing => M.new(:kay => "jewelers"),
             :stuff => true,
           }
         end
@@ -783,11 +797,9 @@ class TestFilters < JekyllUnitTest
       should "successfully group array of Jekyll::Page's" do
         @filter.site.process
         grouping = @filter.group_by(@filter.site.pages, "layout")
+        names = ["default", "nil", ""]
         grouping.each do |g|
-          assert(
-            ["default", "nil", ""].include?(g["name"]),
-            "#{g["name"]} isn't a valid grouping."
-          )
+          assert_includes names, g["name"], "#{g["name"]} isn't a valid grouping."
           case g["name"]
           when "default"
             assert(
@@ -809,7 +821,7 @@ class TestFilters < JekyllUnitTest
               "The list of grouped items for '' is not an Array."
             )
             # adjust array.size to ignore symlinked page in Windows
-            qty = Utils::Platforms.really_windows? ? 14 : 15
+            qty = Utils::Platforms.really_windows? ? 19 : 21
             assert_equal qty, g["items"].size
           end
         end
@@ -825,6 +837,16 @@ class TestFilters < JekyllUnitTest
           )
         end
       end
+
+      should "should pass integers as is" do
+        grouping = @filter.group_by([
+          { "name" => "Allison", "year" => 2016 },
+          { "name" => "Amy", "year" => 2016 },
+          { "name" => "George", "year" => 2019 },
+        ], "year")
+        assert_equal "2016", grouping[0]["name"]
+        assert_equal "2019", grouping[1]["name"]
+      end
     end
 
     context "where filter" do
@@ -842,6 +864,22 @@ class TestFilters < JekyllUnitTest
         assert_equal 2, @filter.where(@array_of_objects, "color", "red").length
       end
 
+      should "filter objects with null properties appropriately" do
+        array = [{}, { "color" => nil }, { "color" => "" }, { "color" => "text" }]
+        assert_equal 2, @filter.where(array, "color", nil).length
+      end
+
+      should "filter objects with numerical properties appropriately" do
+        array = [
+          { "value" => "555" },
+          { "value" => 555 },
+          { "value" => 24.625 },
+          { "value" => "24.625" },
+        ]
+        assert_equal 2, @filter.where(array, "value", 24.625).length
+        assert_equal 2, @filter.where(array, "value", 555).length
+      end
+
       should "filter array properties appropriately" do
         hash = {
           "a" => { "tags"=>%w(x y) },
@@ -860,6 +898,36 @@ class TestFilters < JekyllUnitTest
         assert_equal 2, @filter.where(hash, "tags", "x").length
       end
 
+      should "filter hash properties with null and empty values" do
+        hash = {
+          "a" => { "tags" => {} },
+          "b" => { "tags" => "" },
+          "c" => { "tags" => nil },
+          "d" => { "tags" => ["x", nil] },
+          "e" => { "tags" => [] },
+          "f" => { "tags" => "xtra" },
+        }
+
+        assert_equal [{ "tags" => nil }], @filter.where(hash, "tags", nil)
+
+        assert_equal(
+          [{ "tags" => "" }, { "tags" => ["x", nil] }],
+          @filter.where(hash, "tags", "")
+        )
+
+        # `{{ hash | where: 'tags', empty }}`
+        assert_equal(
+          [{ "tags" => {} }, { "tags" => "" }, { "tags" => nil }, { "tags" => [] }],
+          @filter.where(hash, "tags", Liquid::Expression::LITERALS["empty"])
+        )
+
+        # `{{ `hash | where: 'tags', blank }}`
+        assert_equal(
+          [{ "tags" => {} }, { "tags" => "" }, { "tags" => nil }, { "tags" => [] }],
+          @filter.where(hash, "tags", Liquid::Expression::LITERALS["blank"])
+        )
+      end
+
       should "not match substrings" do
         hash = {
           "a" => { "category"=>"bear" },
@@ -878,12 +946,12 @@ class TestFilters < JekyllUnitTest
 
         results = @filter.where(hash, "featured", "true")
         assert_equal 2, results.length
-        assert_equal 9.2, results[0]["rating"]
-        assert_equal 4.7, results[1]["rating"]
+        assert_in_delta(9.2, results[0]["rating"])
+        assert_in_delta(4.7, results[1]["rating"])
 
         results = @filter.where(hash, "rating", 4.7)
         assert_equal 1, results.length
-        assert_equal 4.7, results[0]["rating"]
+        assert_in_delta(4.7, results[0]["rating"])
       end
 
       should "always return an array if the object responds to 'select'" do
@@ -913,6 +981,45 @@ class TestFilters < JekyllUnitTest
         )
       end
 
+      should "filter objects appropriately with 'or', 'and' operators" do
+        assert_equal(
+          [
+            { "color" => "teal", "size" => "large"  },
+            { "color" => "red",  "size" => "large"  },
+            { "color" => "red",  "size" => "medium" },
+          ],
+          @filter.where_exp(
+            @array_of_objects, "item", "item.color == 'red' or item.size == 'large'"
+          )
+        )
+
+        assert_equal(
+          [
+            { "color" => "red", "size" => "large" },
+          ],
+          @filter.where_exp(
+            @array_of_objects, "item", "item.color == 'red' and item.size == 'large'"
+          )
+        )
+      end
+
+      should "filter objects across multiple conditions" do
+        sample = [
+          { "color" => "teal", "size" => "large", "type" => "variable" },
+          { "color" => "red",  "size" => "large", "type" => "fixed" },
+          { "color" => "red",  "size" => "medium", "type" => "variable" },
+          { "color" => "blue", "size" => "medium", "type" => "fixed" },
+        ]
+        assert_equal(
+          [
+            { "color" => "red", "size" => "large", "type" => "fixed" },
+          ],
+          @filter.where_exp(
+            sample, "item", "item.type == 'fixed' and item.color == 'red' or item.color == 'teal'"
+          )
+        )
+      end
+
       should "stringify during comparison for compatibility with liquid parsing" do
         hash = {
           "The Words" => { "rating" => 1.2, "featured" => false },
@@ -922,12 +1029,12 @@ class TestFilters < JekyllUnitTest
 
         results = @filter.where_exp(hash, "item", "item.featured == true")
         assert_equal 2, results.length
-        assert_equal 9.2, results[0]["rating"]
-        assert_equal 4.7, results[1]["rating"]
+        assert_in_delta(9.2, results[0]["rating"])
+        assert_in_delta(4.7, results[1]["rating"])
 
         results = @filter.where_exp(hash, "item", "item.rating == 4.7")
         assert_equal 1, results.length
-        assert_equal 4.7, results[0]["rating"]
+        assert_in_delta(4.7, results[0]["rating"])
       end
 
       should "filter with other operators" do
@@ -972,8 +1079,209 @@ class TestFilters < JekyllUnitTest
         @filter.site.tap(&:read)
         posts = @filter.site.site_payload["site"]["posts"]
         results = @filter.where_exp(posts, "post",
-          "post.date > site.dont_show_posts_before")
-        assert_equal posts.select { |p| p.date > @sample_time }.count, results.length
+                                    "post.date > site.dont_show_posts_before")
+        assert_equal posts.count { |p| p.date > @sample_time }, results.length
+      end
+    end
+
+    context "find filter" do
+      should "return any input that is not an array" do
+        assert_equal "some string", @filter.find("some string", "la", "le")
+      end
+
+      should "filter objects in a hash appropriately" do
+        hash = { "a" => { "color" => "red" }, "b" => { "color" => "blue" } }
+        assert_equal({ "color" => "red" }, @filter.find(hash, "color", "red"))
+      end
+
+      should "filter objects appropriately" do
+        assert_equal(
+          { "color" => "red", "size" => "large" },
+          @filter.find(@array_of_objects, "color", "red")
+        )
+      end
+
+      should "filter objects with null properties appropriately" do
+        array = [{}, { "color" => nil }, { "color" => "" }, { "color" => "text" }]
+        assert_equal({}, @filter.find(array, "color", nil))
+      end
+
+      should "filter objects with numerical properties appropriately" do
+        array = [
+          { "value" => "555" },
+          { "value" => 555 },
+          { "value" => 24.625 },
+          { "value" => "24.625" },
+        ]
+        assert_equal({ "value" => 24.625 }, @filter.find(array, "value", 24.625))
+        assert_equal({ "value" => "555" }, @filter.find(array, "value", 555))
+      end
+
+      should "filter array properties appropriately" do
+        hash = {
+          "a" => { "tags" => %w(x y) },
+          "b" => { "tags" => ["x"] },
+          "c" => { "tags" => %w(y z) },
+        }
+        assert_equal({ "tags" => %w(x y) }, @filter.find(hash, "tags", "x"))
+      end
+
+      should "filter array properties alongside string properties" do
+        hash = {
+          "a" => { "tags" => %w(x y) },
+          "b" => { "tags" => "x" },
+          "c" => { "tags" => %w(y z) },
+        }
+        assert_equal({ "tags" => %w(x y) }, @filter.find(hash, "tags", "x"))
+      end
+
+      should "filter hash properties with null and empty values" do
+        hash = {
+          "a" => { "tags" => {} },
+          "b" => { "tags" => "" },
+          "c" => { "tags" => nil },
+          "d" => { "tags" => ["x", nil] },
+          "e" => { "tags" => [] },
+          "f" => { "tags" => "xtra" },
+        }
+
+        assert_equal({ "tags" => nil }, @filter.find(hash, "tags", nil))
+        assert_equal({ "tags" => "" }, @filter.find(hash, "tags", ""))
+
+        # `{{ hash | find: 'tags', empty }}`
+        assert_equal(
+          { "tags" => {} },
+          @filter.find(hash, "tags", Liquid::Expression::LITERALS["empty"])
+        )
+
+        # `{{ `hash | find: 'tags', blank }}`
+        assert_equal(
+          { "tags" => {} },
+          @filter.find(hash, "tags", Liquid::Expression::LITERALS["blank"])
+        )
+      end
+
+      should "not match substrings" do
+        hash = {
+          "a" => { "category" => "bear" },
+          "b" => { "category" => "wolf" },
+          "c" => { "category" => %w(bear lion) },
+        }
+        assert_nil @filter.find(hash, "category", "ear")
+      end
+
+      should "stringify during comparison for compatibility with liquid parsing" do
+        hash = {
+          "The Words" => { "rating" => 1.2, "featured" => false },
+          "Limitless" => { "rating" => 9.2, "featured" => true },
+          "Hustle"    => { "rating" => 4.7, "featured" => true },
+        }
+
+        result = @filter.find(hash, "featured", "true")
+        assert_in_delta(9.2, result["rating"])
+
+        result = @filter.find(hash, "rating", 4.7)
+        assert_in_delta(4.7, result["rating"])
+      end
+    end
+
+    context "find_exp filter" do
+      should "return any input that is not an array" do
+        assert_equal "some string", @filter.find_exp("some string", "la", "le")
+      end
+
+      should "filter objects in a hash appropriately" do
+        hash = { "a" => { "color"=>"red" }, "b" => { "color"=>"blue" } }
+        assert_equal(
+          { "color" => "red" },
+          @filter.find_exp(hash, "item", "item.color == 'red'")
+        )
+      end
+
+      should "filter objects appropriately" do
+        assert_equal(
+          { "color" => "red", "size" => "large" },
+          @filter.find_exp(@array_of_objects, "item", "item.color == 'red'")
+        )
+      end
+
+      should "filter objects appropriately with 'or', 'and' operators" do
+        assert_equal(
+          { "color" => "teal", "size" => "large" },
+          @filter.find_exp(
+            @array_of_objects, "item", "item.color == 'red' or item.size == 'large'"
+          )
+        )
+
+        assert_equal(
+          { "color" => "red", "size" => "large" },
+          @filter.find_exp(
+            @array_of_objects, "item", "item.color == 'red' and item.size == 'large'"
+          )
+        )
+      end
+
+      should "filter objects across multiple conditions" do
+        sample = [
+          { "color" => "teal", "size" => "large", "type" => "variable" },
+          { "color" => "red",  "size" => "large", "type" => "fixed" },
+          { "color" => "red",  "size" => "medium", "type" => "variable" },
+          { "color" => "blue", "size" => "medium", "type" => "fixed" },
+        ]
+        assert_equal(
+          { "color" => "red", "size" => "large", "type" => "fixed" },
+          @filter.find_exp(
+            sample, "item", "item.type == 'fixed' and item.color == 'red' or item.color == 'teal'"
+          )
+        )
+      end
+
+      should "stringify during comparison for compatibility with liquid parsing" do
+        hash = {
+          "The Words" => { "rating" => 1.2, "featured" => false },
+          "Limitless" => { "rating" => 9.2, "featured" => true },
+          "Hustle"    => { "rating" => 4.7, "featured" => true },
+        }
+
+        result = @filter.find_exp(hash, "item", "item.featured == true")
+        assert_in_delta(9.2, result["rating"])
+
+        result = @filter.find_exp(hash, "item", "item.rating == 4.7")
+        assert_in_delta(4.7, result["rating"])
+      end
+
+      should "filter with other operators" do
+        assert_equal 3, @filter.find_exp([1, 2, 3, 4, 5], "n", "n >= 3")
+      end
+
+      objects = [
+        { "id" => "a", "groups" => [1, 2] },
+        { "id" => "b", "groups" => [2, 3] },
+        { "id" => "c" },
+        { "id" => "d", "groups" => [1, 3] },
+      ]
+      should "filter with the contains operator over arrays" do
+        result = @filter.find_exp(objects, "obj", "obj.groups contains 1")
+        assert_equal "a", result["id"]
+      end
+
+      should "filter with the contains operator over hash keys" do
+        result = @filter.find_exp(objects, "obj", "obj contains 'groups'")
+        assert_equal "a", result["id"]
+      end
+
+      should "filter posts" do
+        site = fixture_site.tap(&:read)
+        posts = site.site_payload["site"]["posts"]
+        result = @filter.find_exp(posts, "obj", "obj.title == 'Foo Bar'")
+        assert_equal(result, site.posts.find { |p| p.title == "Foo Bar" })
+      end
+
+      should "filter by variable values" do
+        @filter.site.tap(&:read)
+        posts  = @filter.site.site_payload["site"]["posts"]
+        result = @filter.find_exp(posts, "post", "post.date > site.dont_show_posts_before")
+        assert result.date > @sample_time
       end
     end
 
@@ -981,11 +1289,9 @@ class TestFilters < JekyllUnitTest
       should "successfully group array of Jekyll::Page's" do
         @filter.site.process
         groups = @filter.group_by_exp(@filter.site.pages, "page", "page.layout | upcase")
+        names = ["DEFAULT", "NIL", ""]
         groups.each do |g|
-          assert(
-            ["DEFAULT", "NIL", ""].include?(g["name"]),
-            "#{g["name"]} isn't a valid grouping."
-          )
+          assert_includes names, g["name"], "#{g["name"]} isn't a valid grouping."
           case g["name"]
           when "DEFAULT"
             assert(
@@ -1007,7 +1313,7 @@ class TestFilters < JekyllUnitTest
               "The list of grouped items for '' is not an Array."
             )
             # adjust array.size to ignore symlinked page in Windows
-            qty = Utils::Platforms.really_windows? ? 14 : 15
+            qty = Utils::Platforms.really_windows? ? 19 : 21
             assert_equal qty, g["items"].size
           end
         end
@@ -1074,10 +1380,6 @@ class TestFilters < JekyllUnitTest
       end
       should "return sorted strings" do
         assert_equal %w(10 2), @filter.sort(%w(10 2))
-        assert_equal(
-          [{ "a" => "10" }, { "a" => "2" }],
-          @filter.sort([{ "a" => "10" }, { "a" => "2" }], "a")
-        )
         assert_equal %w(FOO Foo foo), @filter.sort(%w(foo Foo FOO))
         assert_equal %w(_foo foo foo_), @filter.sort(%w(foo_ _foo foo))
         # Cyrillic
@@ -1088,7 +1390,19 @@ class TestFilters < JekyllUnitTest
       end
       should "return sorted by property array" do
         assert_equal [{ "a" => 1 }, { "a" => 2 }, { "a" => 3 }, { "a" => 4 }],
-          @filter.sort([{ "a" => 4 }, { "a" => 3 }, { "a" => 1 }, { "a" => 2 }], "a")
+                     @filter.sort([{ "a" => 4 }, { "a" => 3 }, { "a" => 1 }, { "a" => 2 }], "a")
+      end
+      should "return sorted by property array with numeric strings sorted as numbers" do
+        assert_equal([{ "a" => ".5" }, { "a" => "0.65" }, { "a" => "10" }],
+                     @filter.sort([{ "a" => "10" }, { "a" => ".5" }, { "a" => "0.65" }], "a"))
+      end
+      should "return sorted by property array with numeric strings first" do
+        assert_equal([{ "a" => ".5" }, { "a" => "0.6" }, { "a" => "twelve" }],
+                     @filter.sort([{ "a" => "twelve" }, { "a" => ".5" }, { "a" => "0.6" }], "a"))
+      end
+      should "return sorted by property array with numbers and strings " do
+        assert_equal([{ "a" => "1" }, { "a" => "1abc" }, { "a" => "20" }],
+                     @filter.sort([{ "a" => "20" }, { "a" => "1" }, { "a" => "1abc" }], "a"))
       end
       should "return sorted by property array with nils first" do
         ary = [{ "a" => 2 }, { "b" => 1 }, { "a" => 1 }]
@@ -1097,13 +1411,13 @@ class TestFilters < JekyllUnitTest
       end
       should "return sorted by property array with nils last" do
         assert_equal [{ "a" => 1 }, { "a" => 2 }, { "b" => 1 }],
-          @filter.sort([{ "a" => 2 }, { "b" => 1 }, { "a" => 1 }], "a", "last")
+                     @filter.sort([{ "a" => 2 }, { "b" => 1 }, { "a" => 1 }], "a", "last")
       end
       should "return sorted by subproperty array" do
         assert_equal [{ "a" => { "b" => 1 } }, { "a" => { "b" => 2 } },
                       { "a" => { "b" => 3 } },],
-          @filter.sort([{ "a" => { "b" => 2 } }, { "a" => { "b" => 1 } },
-                        { "a" => { "b" => 3 } },], "a.b")
+                     @filter.sort([{ "a" => { "b" => 2 } }, { "a" => { "b" => 1 } },
+                                   { "a" => { "b" => 3 } },], "a.b")
       end
     end
 
@@ -1131,7 +1445,7 @@ class TestFilters < JekyllUnitTest
 
     context "inspect filter" do
       should "return a HTML-escaped string representation of an object" do
-        assert_equal "{&quot;&lt;a&gt;&quot;=&gt;1}", @filter.inspect({ "<a>" => 1 })
+        assert_equal "{&quot;&lt;a&gt;&quot;=&gt;1}", @filter.inspect("<a>" => 1)
       end
 
       should "quote strings" do
@@ -1202,5 +1516,27 @@ class TestFilters < JekyllUnitTest
         end
       end
     end
+
+    context "number_of_words filter" do
+      should "return the number of words for Latin-only text" do
+        assert_equal 5, @filter.number_of_words("hello world and taoky strong!", "auto")
+        assert_equal 5, @filter.number_of_words("hello world and taoky strong!", "cjk")
+      end
+
+      should "return the number of characters for CJK-only text" do
+        assert_equal 17, @filter.number_of_words("こんにちは、世界！안녕하세요 세상!", "auto")
+        assert_equal 17, @filter.number_of_words("こんにちは、世界！안녕하세요 세상!", "cjk")
+      end
+
+      should "process Latin and CJK independently" do
+        # Intentional: No space between Latin and CJK
+        assert_equal 6, @filter.number_of_words("你好hello世界world", "auto")
+        assert_equal 6, @filter.number_of_words("你好hello世界world", "cjk")
+      end
+
+      should "maintain original behavior unless specified" do
+        assert_equal 1, @filter.number_of_words("你好hello世界world")
+      end
+    end
   end
 end
